#!/usr/bin/python3
import sys
from PyQt5 import QtCore, QtGui, QtWidgets

num_displaycols = 16
num_displayrows = 24

def read_rom(filename):
    with open(filename, 'rb') as f:
        ines = f.read()

    prg = ines[16:]
    return [prg[i:i+16] for i in range(0, len(prg), 16)]


def parse_tile(tile):
    lowplane, hiplane = tile[0:8], tile[8:16]
    rows = []
    for lowbyte, hibyte in zip(lowplane, hiplane):
        row = [
            ((lowbyte >> idx) & 1) + 2*((hibyte >> idx) & 1)
            for idx in range(7, -1, -1)
        ]
        rows.append(row)
    return rows


def layout_tiles(tiles):
    numcols = num_displaycols
    numrows = len(tiles)//numcols

    out = []
    for rowidx in range(0, 8*numrows):
        out.append([0]*(8*numcols))

    for rowidx in range(0, numrows):
        for colidx in range(0, numcols):
            tile = parse_tile(tiles[colidx + numcols * rowidx])
            for i in range(0, 8):
                for j in range(0, 8):
                    out[i + 8*rowidx][j + 8*colidx] = tile[i][j]
    return out


class Canvas(QtWidgets.QLabel):

    def __init__(self):
        super().__init__()

        self.scalefactor = 5
        self.numcols = num_displaycols*8
        self.numrows = num_displayrows*8

        self.file_name = QtWidgets.QFileDialog.getOpenFileName(
            self,
            "Open ROM", "/home/jordi", "NES Files (*.nes)"
        )
        self.laid_tiles = layout_tiles(read_rom(self.file_name[0]))

        offset = self.numrows//2
        self.data = bytes(sum(self.laid_tiles[offset:offset+self.numrows], []))
        image = QtGui.QImage(self.data, self.numcols, self.numrows, QtGui.QImage.Format_Indexed8)
        image.setColorTable([
            QtGui.QColor("#FFFFFF").rgb(),
            QtGui.QColor("#000000").rgb(),
            QtGui.QColor("#8888FF").rgb(),
            QtGui.QColor("#0000FF").rgb(),
        ])
        pixmap = QtGui.QPixmap(image)
        #pixmap.fill(QtGui.QColor("white"))
        self.setPixmap(pixmap.scaled(QtCore.QSize(self.scalefactor*self.numcols, self.scalefactor*self.numrows)))

        self.last_x, self.last_y = None, None
        self.set_pen_color('#000000')

    def set_pen_color(self, c):
        self.pen_color = QtGui.QColor(c)

    def mouseMoveEvent(self, e):
        if self.last_x is None:  # First event.
            self.last_x = e.x()
            self.last_y = e.y()
            return  # Ignore the first time.

        painter = QtGui.QPainter(self.pixmap())
        # painter.scale(self.scalefactor, self.scalefactor)
        p = painter.pen()
        p.setWidth(1)
        p.setColor(self.pen_color)
        painter.setPen(p)
        painter.drawLine(self.last_x, self.last_y, e.x(), e.y())
        painter.end()
        self.update()

        # Update the origin for next time.
        self.last_x = e.x()
        self.last_y = e.y()

    def mouseReleaseEvent(self, e):
        self.last_x = None
        self.last_y = None


COLORS = [
    # 17 undertones https://lospec.com/palette-list/17undertones
    '#000000', '#141923', '#414168', '#3a7fa7', '#35e3e3', '#8fd970', '#5ebb49',
    '#458352', '#dcd37b', '#fffee5', '#ffd035', '#cc9245', '#a15c3e', '#a42f3b',
    '#f45b7a', '#c24998', '#81588d', '#bcb0c2', '#ffffff',
]


class QPaletteButton(QtWidgets.QPushButton):

    def __init__(self, color):
        super().__init__()
        self.setFixedSize(QtCore.QSize(24, 24))
        self.color = color
        self.setStyleSheet("background-color: %s;" % color)


class MainWindow(QtWidgets.QMainWindow):

    def __init__(self):
        super().__init__()

        self.canvas = Canvas()

        w = QtWidgets.QWidget()
        l = QtWidgets.QVBoxLayout()
        w.setLayout(l)
        l.addWidget(self.canvas)

        palette = QtWidgets.QHBoxLayout()
        self.add_palette_buttons(palette)
        l.addLayout(palette)

        self.setCentralWidget(w)

    def add_palette_buttons(self, layout):
        for c in COLORS:
            b = QPaletteButton(c)
            b.pressed.connect(lambda c=c: self.canvas.set_pen_color(c))
            layout.addWidget(b)


app = QtWidgets.QApplication(sys.argv)
window = MainWindow()
window.show()
app.exec_()
